SystemOrganization addCategory: #'Zinc-Seaside'!

WAServerAdaptor subclass: #ZnZincServerAdaptor
	instanceVariableNames: 'server'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-Seaside'!

!ZnZincServerAdaptor commentStamp: '<historical>' prior: 0!
I convert between Seaside and Zinc HTTP requests and responses.

Instance Variables:
	server	<ZnServer>!

!ZnZincServerAdaptor methodsFor: 'private' stamp: 'SvenVanCaekenberghe 9/30/2010 17:15'!
basicStart
	(server := ZnServer on: self port)
		delegate: (ZnSeasideServerAdaptorDelegate with: self);
		start! !

!ZnZincServerAdaptor methodsFor: 'private' stamp: 'pmm 9/8/2010 16:50'!
basicStop
	server isNil
		ifFalse: [ server stop ].
	server := nil! !

!ZnZincServerAdaptor methodsFor: 'private' stamp: 'SvenVanCaekenberghe 2/27/2011 20:19'!
convertMultipart: entity
	| dictionary |
	dictionary := ZnMultiValueDictionary new.
	entity partsDo: [ :each | | value |
		value := each fileName isEmptyOrNil 
			ifTrue: [ each fieldValueString ] 
			ifFalse: [ self convertMultipartFileField: each ] .
		dictionary at: each fieldName put: value ].
	^ dictionary 
		! !

!ZnZincServerAdaptor methodsFor: 'private' stamp: 'SvenVanCaekenberghe 2/27/2011 20:25'!
convertMultipartFileField: part  
	| file |
	(file := WAFile new)
		fileName: part fileName;
		contentType: part contentType printString;
		contents: part contents.
	^ file! !

!ZnZincServerAdaptor methodsFor: 'defaults' stamp: 'pmm 9/8/2010 16:54'!
defaultCodec
	^ GRCodec forEncoding: 'utf-8'! !

!ZnZincServerAdaptor methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 9/9/2010 20:37'!
isRunning
	^ server notNil and: [ server isRunning ]! !

!ZnZincServerAdaptor methodsFor: 'testing' stamp: 'pmm 9/8/2010 16:51'!
isStarting
	^ false ! !

!ZnZincServerAdaptor methodsFor: 'testing' stamp: 'pmm 9/9/2010 22:07'!
isStopped
	^ server isNil or: [ server isRunning not ]! !

!ZnZincServerAdaptor methodsFor: 'testing' stamp: 'pmm 9/8/2010 16:52'!
isStopping
	^ false ! !

!ZnZincServerAdaptor methodsFor: 'converting' stamp: 'pmm 9/9/2010 22:08'!
requestAddressFor: aZincRequest
	^ aZincRequest  headers
		at: ZnConstants remoteAddressHeader 
		ifAbsent: [ nil ]! !

!ZnZincServerAdaptor methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 9/9/2010 20:25'!
requestBodyFor: aZincRequest
	^ aZincRequest entity! !

!ZnZincServerAdaptor methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 2/27/2011 20:23'!
requestCookiesFor: aZincRequest
	| cookieString |
	cookieString := aZincRequest headers
		at: 'Cookie' ifAbsent: [ ^ #() ].
	^ WARequestCookie fromString: cookieString! !

!ZnZincServerAdaptor methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 2/27/2011 20:09'!
requestFieldsFor: aZincRequest
	| fields |
	fields := WARequestFields new.
	aZincRequest method = #POST
		ifTrue: [ 
			aZincRequest entity contentType = ZnMimeType applicationFormUrlEncoded
				ifTrue: [
					fields addAll: aZincRequest entity fields ].
			aZincRequest entity contentType = ZnMimeType multiPartFormData
				ifTrue: [
					fields addAll: (self convertMultipart: aZincRequest entity) ] ].
	^ fields
	! !

!ZnZincServerAdaptor methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 9/12/2010 10:58'!
requestHeadersFor: aZincRequest
	| fields |
	fields := Dictionary new.
	aZincRequest headersDo: [ :key :value |
		fields at: key asLowercase put: value ].
	^ fields! !

!ZnZincServerAdaptor methodsFor: 'converting' stamp: 'pmm 9/8/2010 17:15'!
requestMethodFor: aZincRequest
	^ aZincRequest method greaseString! !

!ZnZincServerAdaptor methodsFor: 'converting' stamp: 'pmm 9/8/2010 17:18'!
requestUrlFor: aZincRequest
	^ (WAUrl absolute: aZincRequest uri asString)
		decodedWith: self codec.! !

!ZnZincServerAdaptor methodsFor: 'converting' stamp: 'pmm 9/8/2010 17:19'!
requestVersionFor: aZincRequest
	^ WAHttpVersion fromString: aZincRequest requestLine version! !

!ZnZincServerAdaptor methodsFor: 'converting' stamp: 'pmm 9/12/2010 14:41'!
responseFrom: aRequestContext
	| partialHeaders cookies fullHeaders seasideResponse contents entity contentType |
	seasideResponse := aRequestContext response.
	partialHeaders := seasideResponse headers.
	cookies := seasideResponse cookies.
	fullHeaders := ZnHeaders defaultResponseHeaders.
	partialHeaders keysAndValuesDo: [ :key :value |
		fullHeaders at: key put: value ].
	cookies do: [ :each |
		fullHeaders at: 'Set-Cookie' add: each oldNetscapeString.
		fullHeaders at: 'Set-Cookie2' add: each rfc2965String ].
	contentType := seasideResponse contentType greaseString.
	contents := seasideResponse contents.
	entity := ZnEntity with: contents type: contentType.
	^ ZnResponse new
		statusLine: (ZnStatusLine code: seasideResponse status);
		headers: fullHeaders;
		entity: entity;
		yourself! !

!ZnZincServerAdaptor methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/9/2010 20:38'!
statusString
	^ 'zinc on port ', self port greaseString, 
		( self isRunning ifTrue: [ ' [running]' ] ifFalse: [ ' [stopped]' ] )! !

Object subclass: #ZnSeasideServerAdaptorDelegate
	instanceVariableNames: 'adaptor'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-Seaside'!

!ZnSeasideServerAdaptorDelegate commentStamp: 'SvenVanCaekenberghe 9/30/2010 18:13' prior: 0!
I am ZnSeasideServerAdaptorDelegate.

I handle Zinc HTTP Server delegate's #handleRequest: using WAServerAdaptor>>#process:

Instance Variables:
	adaptor	<WAServerAdaptor>!

!ZnSeasideServerAdaptorDelegate class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/30/2010 17:14'!
with: aSeasideServerAdaptor
	^ self new
		adaptor: aSeasideServerAdaptor;
		yourself! !

!ZnSeasideServerAdaptorDelegate methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/30/2010 17:13'!
adaptor
	^ adaptor ! !

!ZnSeasideServerAdaptorDelegate methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/30/2010 17:12'!
adaptor: aSeasideServerAdaptor
	adaptor := aSeasideServerAdaptor ! !

!ZnSeasideServerAdaptorDelegate methodsFor: 'public' stamp: 'SvenVanCaekenberghe 9/30/2010 18:13'!
handleRequest: znRequest
	"I pass a Zinc HTTP Components ZnRequest to my Seaside Server Adaptor using #process:"
	
	^ self adaptor process: znRequest! !
